/**@name LIBRARY: unSystemIntegrity/unSystemIntegrity_alarmDescription.ctl

@author: Juan Manuel Pablo Rodriguez

Creation Date: 17/05/2023

version 1.0

Purpose:
	Set and get descriptions for system integrity alarms.

Usage: Public
*/

//------------------------------------------------------------------------------------------------------------------------
// struct unSystemIntegrity_AlarmDescriptionTimestamp
/**
Purpose:
Timestamps definitions for the unSystemIntegrity alarm description
*/
struct unSystemIntegrity_AlarmDescriptionTimestamp {
  time timeStamp;
  atime aTimeStamp;
  string dpeAlert;
  int alarmValue;
};

//------------------------------------------------------------------------------------------------------------------------
// class unSystemIntegrity_UpdateAlarmDescription
/**
Purpose:
Handle the update of alarm descriptions by using the class event trigger mechanism
*/

class unSystemIntegrity_UpdateAlarmDescription {
  #event updateAlarmDescription(time timeStamp, atime aTimeStamp, string dpeAlert, int alarmValue)

  public void triggerUpdateAlarmDesc(time timeStamp, atime aTimeStamp, string dpeAlert, int alarmValue) {
    triggerClassEvent(this.updateAlarmDescription, timeStamp, aTimeStamp, dpeAlert, alarmValue);
  }
};

//------------------------------------------------------------------------------------------------------------------------
// unSystemIntegrity_registerAlarmDescription
/**
Purpose:
Registers the alarm description and triggers the callback function whenever there is a change in the alert value.

	@param dpeAlarm1: string, input, alarm datapoint element
 @param updAlDesc: shared_ptr, input, description pointer for the system integrity type
 @param exceptionInfo: dyn_string, output, the error is returned here
*/

void unSystemIntegrity_registerAlarmDescription(string dpeAlarm1, shared_ptr<unSystemIntegrity_UpdateAlarmDescription> updAlDesc, dyn_string &exceptionInfo) {
  string dpeAlarm = dpSubStr(dpeAlarm1, DPSUB_DP_EL);

  if (!dpExists(dpeAlarm)) {
    return;
  }

  string query = "SELECT ALERT '_alert_hdl.._value' FROM '" + dpeAlarm + "'";
  int rc = dpQueryConnectSingle("unSystemIntegrity_updateAlarmDescriptionCB", TRUE, updAlDesc, query, 500);
  dyn_errClass err = getLastError();
  if (dynlen(err)) {
    fwException_raise(exceptionInfo, "ERROR", "unSystemIntegrity_registerAlarmDescription(): dpQueryConnectSingle has returned an error. " + err,"");
  }
}

//------------------------------------------------------------------------------------------------------------------------
// unSystemIntegrity_updateAlarmDescriptionCB
/**
Purpose:
Callback function that triggers the update alarm description when the alert value has changed

          @param userData: anytype, input, user-defined data passed as parameter in the query
          @param alertInfo: dyn_dyn_anytype, input, alert information value coming from the query

*/

void unSystemIntegrity_updateAlarmDescriptionCB(anytype userData, dyn_dyn_anytype alertInfo) {
  unSystemIntegrity_AlarmDescriptionTimestamp myTimeStamp;
  int lenResults = dynlen(alertInfo);
  if (lenResults >= 1) {
    for (int i = 2; i <= lenResults; i++) {
      myTimeStamp.timeStamp = alertInfo[i][2];
      myTimeStamp.aTimeStamp = alertInfo[i][2];
      myTimeStamp.dpeAlert = getAIdentifier(alertInfo[i][2]);
      myTimeStamp.alarmValue = alertInfo[i][3];
      if (!dpExists(myTimeStamp.dpeAlert)) {
        DebugTN(__FUNCTION__, __LINE__, "Alert datapoint element " + myTimeStamp.dpeAlert + " does not exist or has been removed.");
	   return;
      }
    }
  }
  userData.triggerUpdateAlarmDesc(myTimeStamp.timeStamp, myTimeStamp.aTimeStamp, myTimeStamp.dpeAlert, myTimeStamp.alarmValue);
}

//------------------------------------------------------------------------------------------------------------------------
// unSystemIntegrity_setAlarmDescription
/**
Purpose:
Add a short description to a system integrity alarm

          @param alarmDescription: string, input, description to be set
          @param timeStamp         input, time, alert timestamp
          @param aTimeStamp        input, atime, alert atimestamp
          @param dpeAlert          input, string, alert datapoint element
*/

void unSystemIntegrity_setAlarmDescription(string alarmDescription, time timeStamp, atime aTimeStamp, string dpeAlert) {
  if (dpExists(dpeAlert)) {
    int rc = alertSetWait(timeStamp, getACount(aTimeStamp), dpeAlert + "._comment", alarmDescription);
    dyn_errClass err = getLastError();
    if (dynlen(err)) {
      DebugTN(__FUNCTION__, __LINE__, "Alert comment cannot be written to datapoint element " + dpeAlert + "._comment", err);
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------
// unSystemIntegrity_getAlarmDescription
/**
Purpose:
Obtain a short description of a system integrity alarm

     @param dpeAlarm1: string, input, alarm datapoint element
     @param exceptionInfo: dyn_string, output, the error is returned here

     @return string with the description of the alarm

*/
string unSystemIntegrity_getAlarmDescription(string dpeAlarm1, dyn_string &exceptionInfo) {
  string alarmDescription;
  int range;

  string dpeAlarm = dpSubStr(dpeAlarm1, DPSUB_DP_EL);
  if (!dpExists(dpeAlarm)) {
   fwException_raise(exceptionInfo, "ERROR", "unSystemIntegrity_getAlarmDescription(): Alarm datapoint element " + dpeAlarm + " does not exist.", "");
	  return alarmDescription;
  }

  int alarmValue;
  int rc = dpGet(dpeAlarm, alarmValue);
  dyn_errClass err = getLastError();
  if (dynlen(err)) {
   fwException_raise(exceptionInfo, "ERROR", "unSystemIntegrity_getAlarmDescription(): Alarm value from  " + dpeAlarm + " cannot be obtained. " + err, "");
	  return alarmDescription;
  }

  if(alarmValue == 0) {
    return alarmDescription;
  }

  int rc = dpGet(dpeAlarm+":_alert_hdl.._act_range", range);
  dyn_errClass err = getLastError();
  if (dynlen(err)) {
   fwException_raise(exceptionInfo, "ERROR", "unSystemIntegrity_getAlarmDescription(): Alert range from " + dpeAlarm + ":_alert_hdl.._act_range cannot be obtained. " + err, "");
	  return alarmDescription;
  }

  dyn_string dt;
  rc = dpGet(dpeAlarm + ":_alert_hdl." + range + "._alerts", dt);
  err = getLastError();
  if (dynlen(err)) {
   fwException_raise(exceptionInfo, "ERROR", "unSystemIntegrity_getAlarmDescription(): Alert time from " + dpeAlarm + ":_alert_hdl." + range + "._alerts cannot be obtained. " + err, "");
	  return alarmDescription;
  }

  if (dynlen(dt) < 1) {
   DebugTN(__FUNCTION__, __LINE__, "WARNING", "unSystemIntegrity_getAlarmDescription(): Alert time from " + dpeAlarm + ":_alert_hdl." + range + "._alerts is empty.");
	  return alarmDescription;
  }

  time timeAlert = dt[dynlen(dt)]; //always take the latest known alert otherwise the comment is only shown for the first time alert
  atime at = makeATime(timeAlert, 0, dpeAlarm + ":_alert_hdl." + range + "._comment");
  rc = alertGet(timeAlert, getACount(at), dpSubStr(getAIdentifier(at), DPSUB_SYS_DP_EL_CONF_DET) + "._comment", alarmDescription);
  err = getLastError();
  if (dynlen(err)) {
   fwException_raise(exceptionInfo, "ERROR", "unSystemIntegrity_getAlarmDescription(): Alert comment cannot be read from datapoint element " + dpeAlarm + ":_alert_hdl." + range + "._comment. " + err, "");
  }

  return alarmDescription;
}
